#!/usr/bin/env python

from twisted.internet import gtk2reactor # extra dep pygtk2
gtk2reactor.install()

import gtk, gobject

from twisted.internet import reactor, protocol, defer # extra dep twisted
from twisted.protocols import basic

import ConfigParser
import os
import time
import binascii
from Crypto.Cipher import Blowfish  # Extra dep pycrypto

class CrypTo:
	def __init__(self,key):
		self.crypt = Blowfish.new(key)
		
	def encrypt(self, line):
		line = self.fixlength(line)
		line = self.crypt.encrypt(line)
		line = binascii.b2a_hex(line)
		return line
			
	def decrypt(self, line):
		# Guess what i decrypt stuff.
		line = binascii.a2b_hex(line.strip())
		line = self.crypt.decrypt(line)
		line = line.replace('\n','') # Remove extra chars
		data = line.split('\r')
		return data

	def fixlength(self, line):
		# I will fix the length of the string.
		# So we wont have any odders ;)
		length = len(line)
		for max in range(1, 51):
			if length <= 8*max:
				add = (8*max) - length
				break
		extra = '\n'
		line += extra*add
		return line
		
class MenotProtocol(basic.LineReceiver):
	delimiter = '\n'

	def connectionMade(self):
		print "connected to %s" % host
		if show_connect == 1:
			self.send("%s enters the cave" % nick,0)
		
	def lineReceived(self, line):
		self.handler(line)

	def connectionLost(self, reason):
		print "lost connection"

	def handler(self, line):
		line = crypt.decrypt(line)
		if line[0] == "00": # Handle public bubble
			t = time.strftime("%H:%M")
			msg = "[%s] %s" % (t, line[1])
			window.show_data(msg)
		elif line[0] == "01": # Handle wisper
			cryp = CrypTo(nick)
			msg = line[1]
			msg = cryp.decrypt(msg)
			msg = msg[0]
			if msg[:1] == '+':
				t = time.strftime("%H:%M")
				msg = "{%s} %s" %(t, msg[1:])
				window.show_data(msg)

	def send(self, line):
		line = crypt.encrypt(line)
		if (len(line)%2)==0:
			self.sendLine(line)

class MenotFactory(protocol.ClientFactory):
	def buildProtocol(self, addr):
		global protocol
		protocol = MenotProtocol()
		return protocol

	def clientConnectionFailed(self, connectior, reason):
		print "Connection failed:", reason.getErrorMessage()
		reactor.stop()

	def clientConnectionLost(self, connectior, reason):
		print "Connection lost:", reason.getErrorMessage()
		reactor.stop()

	def send(self, line, type, to="none"):
		if type == 0:
			line = "0%i\r%s: %s" %(type, nick, line)
			protocol.send(line)
		elif type == 1:
			line = "+%s: %s" %(nick, line)
			cryp = CrypTo(to)
			line = cryp.encrypt(line)
			line = "0%i\r%s" %(type, line)
			protocol.send(line)

class MainWindow(gtk.Window):
	def __init__(self):
		# create the window and its widgets
		gtk.Window.__init__(self)
		self.connect('destroy', self.destroy)
		self.table = gtk.Table(rows=2, columns=2, homogeneous=False)
		self.buffer = gtk.TextBuffer()
		self.log = gtk.TextView(self.buffer)
		self.log.set_editable(0)
		self.scroll = gtk.ScrolledWindow()
		self.scroll.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)
		self.scroll.add(self.log)
		self.adjust = gtk.Adjustment()
		self.scroll.set_vadjustment(self.adjust)
		self.table.attach(self.scroll, 0, 1, 0, 1)
		self.input = gtk.Entry()
		self.connect('key-press-event', self.key_press)
		self.table.attach(self.input, 0, 2, 1, 2, xoptions=gtk.FILL, yoptions=gtk.FILL)
		self.add(self.table)
		self.set_default_size(296, 279)
		self.set_title("menot-node: %s" % nick)
		self.set_border_width(5)
		self.show_all()
		
	def destroy(self, widget, data=None):
		#factory.send("%s left the cave" % nick) # dont work anyway
		reactor.stop()
		gtk.main_quit()
		os._exit(0)

	def show_data(self, data):
		# add the text to buffer
		window.buffer.insert(window.buffer.get_end_iter(), data + '\n')
		# scroll to buttom (so we dont miss the show)
		window.adjust.value = window.adjust.upper

	def key_press(self, widget, event):
		# If enter was pressed send the data ;)
		# later settings could be changed from this point to
		if event.keyval == gtk.keysyms.Return:
			text = self.input.get_text()
			if text == "/quit":
				# quit!
				self.destroy('window')
			if text[:7] == '/wisper' or text[:4] == '/msg' or text[:3] == '/pm':
				# send a wisper
				cmd,to,msg = text.split(' ',2)
				if cmd and to and msg:
					factory.send(msg,1,to)
			else:
				factory.send(text,0)
			self.input.set_text("")

if __name__ == '__main__':
	global window
	global factory

	config = ConfigParser.ConfigParser()
	config.read("./menot-client.conf")
	host = config.get("server","host")
	port = config.getint("server","port")
	nick = config.get("user","nick")
	key = config.get("user","key")
	show_connect = config.getint("user","show_connect")

	crypt = CrypTo(key)
	window = MainWindow()
	factory = MenotFactory()
	reactor.connectTCP(host, port, factory)
	reactor.run()
